﻿using System;
using System.Runtime.InteropServices;

namespace XFUDriver
{
    class XFUDriver
    {
        #region enum
        public enum ECM_SLV_INFO : byte
        {
            vid,
            pid,
            rev,
            name,
            configadr,
            aliasadr,
            state,
            ALstatuscode,
            Obytes,
            Ibytes,
            hasdc,
            DCcycle,
            DCshift,
            DCactive,
            outputs,
            inputs,
            islost,
            eep_8byte,
            Itype,
            Dtype,
            Obits,
            Ostartbit,
            Ibits,
            Istartbit,
            ptype,
            topology,
            pdelay,
            DCnext,
            DCprevious,
            configindex,
            SIIindex,
            eep_pdi,
            CoEdetails,
            FoEdetails,
            EoEdetails,
            SoEdetails,
            Ebuscurrent,
            blockLRW,
            group,
            FMMUunused,
            mbx_proto
        };

        public enum CIA402_SW : byte
        {
            NOTREADYTOSWITCHON = 0x00,
            QUICKSTOPACTIVE = 0x07,
            FAULT = 0x08,
            FAULTREACTIONACTIVE = 0x0F,
            READYTOSWITCHON = 0x21,
            SWITCHEDON = 0x23,
            OPERATIONENABLED = 0x27,
            SWITCHEDONDISABLED = 0x40,
            STATE_MASK = 0x6F
        };

        public enum ECM_PDO : byte
        {
            WR_OP = 0x05,
            RD_OP = 0x06,
            RW_OP = 0x07,
        };

        public enum ECM_CRC_TYPE : byte
        {
            NONE = 0,
            TYPE_32 = 3,
        };

        public enum ecm_datagram_commad : byte
        {
            ECM_RAW_API_APRD = 1,
            ECM_RAW_API_APWR = 2,
            ECM_RAW_API_FPRD = 4,
            ECM_RAW_API_FPWR = 5,
            ECM_RAW_API_BRD = 7,
            ECM_RAW_API_BWR = 8,
            ECM_RAW_API_LRD = 10,
            ECM_RAW_API_LWR = 11,
            ECM_RAW_API_LRW = 12,
            ECM_RAW_API_ARMW = 13,
            ECM_RAW_API_FRMW = 14,
        }
        #endregion



        #region Struct
        [StructLayout(LayoutKind.Sequential)]
        public struct OBJ_ENTRY_T
        {
            public byte u8BitSize;
            public byte u8SubIdx;
            public UInt16 u16Idx;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct PDO_CONFIG_HEAD
        {
            public byte slave;
            public byte PDOCnt;
            public UInt16 SmaIdx;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
            public UInt16[] MapIdx;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
            public UInt16[] ObjsCnt;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 24)]
            public OBJ_ENTRY_T[,] Table;
        }
        #endregion

        #region Define
        public const byte EC_STATE_NONE = 0x00;
        public const byte EC_STATE_INIT = 0x01;
        public const byte EC_STATE_PRE_OP = 0x02;
        public const byte EC_STATE_BOOT = 0x03;
        public const byte EC_STATE_SAFE_OP = 0x04;
        public const byte EC_STATE_OPERATIONAL = 0x08;

        public const int PKG_DATA_DEFAULT_SIZE = 992;

        public const byte ECM_FIFO_WR = 1;
        public const byte ECM_FIFO_RD = 2;
        public const byte ECM_FIFO_WR_DIR = 4;
        public const byte ECM_FIFO_RD_DIR = 8;
        public const byte ECM_FIFO_RW = 3;
        public const byte ECM_FIFO_RWD = 6;
        public const byte ECM_FIFO_RDW = 9;

        public const UInt16 ECM_EEPROM_OP_RD = 0;
        public const UInt16 ECM_EEPROM_OP_WR = 1;
        #endregion

        #region Function
        internal const string DrvDllFilePath = @"EcmXFUDrv.dll";
        internal const string PlatformFilePath = @"ECMPLATFORM.dll";
        #region Platform function
        [DllImport(PlatformFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int XF_SetDevType(int nType);
        [DllImport(PlatformFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int XF_GetDevType();
        [DllImport(PlatformFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int UserSys_Init(UInt32 u32SpiFreq);
        [DllImport(PlatformFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern void UserSys_Close();
        #endregion

        #region System function
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetNetLinkPin(ref int pPinStatus);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern byte ECM_InitLibrary(ref ushort pu16SpiDataSize, ECM_CRC_TYPE u8CrcType);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern void ECM_CloseLibrary();
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatInit(UInt16 DCActCode, UInt32 CycTime);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatReconfig();
        #endregion
        #region Slave Info
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_SlaveInfoGet(byte slave, ECM_SLV_INFO info, ref UInt32 pBuf);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatDatagramReq(ecm_datagram_commad cmd, UInt16 position, UInt16 offset, UInt32 logicalAddress, UInt16 length, UInt32 Timeout, ref UInt32 u32Data);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatDatagramReq(ecm_datagram_commad cmd, UInt16 position, UInt16 offset, UInt32 logicalAddress, UInt16 length, UInt32 Timeout, ref UInt16 u16Data);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatDatagramReq(ecm_datagram_commad cmd, UInt16 position, UInt16 offset, UInt32 logicalAddress, UInt16 length, UInt32 Timeout, ref byte u8Data);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatDatagramGet(ref UInt32 pBuf);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatDatagramGet(ref UInt16 pBuf);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatDatagramGet(ref byte pBuf);

        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern sbyte ECM_EcatSlvCntGet();
        #endregion
        #region FIFO
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_ClearFIFO(byte TxRx);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EnableFIFO(byte enable);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_InitFIFO();
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern ushort ECM_FifoRxPdoSizeGet();
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern ushort ECM_FifoTxPdoSizeGet();
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_SetTxFIFOCnt(byte u8TxCnt);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_SetRxFIFOCnt(byte u8RxCnt);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetTxFIFOCnt(ref byte pu8Cnt);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetRxFIFOCnt(ref byte pu8Cnt);
        #endregion
        #region State Control
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatStateSet(byte u8Slave, byte u8State);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatStateUpdate();
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatStateGet(byte u8Slave, ref byte pu8State);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_StateCheck(byte slave, byte expectState, int timeOutMS);
        #endregion
        #region Driver 402
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_Drv402SM_AdvConfig(byte TblIdx, byte SlvIdx, byte ContrlWordOffset, byte StateWordOffset);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_Drv402SM_StateSet(byte TblIdx, CIA402_SW State);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_Drv402SM_StateGet(byte TblIdx, ref CIA402_SW pState);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_Drv402SM_StateCheck(byte TblIdx, CIA402_SW ExceptState, int TimeOutMS);
        #endregion
        #region PDO
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern byte ECM_EcatPdoDataExchange(ECM_PDO u8OP, ref byte pRxData, ref byte pTxData, ref UInt16 pu16DataSize);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatPdoFifoDataExchange(byte u8FifoThreshold, ref byte pRxData, ref byte pTxData, UInt16 u16DataSize, ref byte pRxPDOFifoCnt, ref byte CrcErrCnt, ref byte WkcErrCnt);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatPdoFifoDataExchangeAdv(byte u8Op, byte u8Cnt, byte[] pRxData, byte[] pTxData, UInt16 u16DataSize, ref byte pu8RxPdoFifoCnt, ref byte CrcErrCnt, ref byte WkcErrCnt, ref byte IsSlvAlive);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatPdoFifoIsFull(byte u8FifoThreshold);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetPDOConfig(int Slave, int SmaIdx, ref PDO_CONFIG_HEAD pPdoConfigBuf);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_ShowPDOConfig(int Slave, int SmaIdx);
        #endregion
        #region SDO
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatSdoWrite(byte Slave, UInt16 Index, byte SubIndex, UInt16 size, int Timeout_ns, ref byte Data);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatSdoWrite(byte Slave, UInt16 Index, byte SubIndex, UInt16 size, int Timeout_ns, ref UInt16 Data);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatSdoWrite(byte Slave, UInt16 Index, byte SubIndex, UInt16 size, int Timeout_ns, ref UInt32 Data);

        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatSdoRead(byte Slave, UInt16 Index, byte SubIndex, UInt16 size, int Timeout_ns, ref byte Data);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatSdoRead(byte Slave, UInt16 Index, byte SubIndex, UInt16 size, int Timeout_ns, ref UInt16 Data);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatSdoRead(byte Slave, UInt16 Index, byte SubIndex, UInt16 size, int Timeout_ns, ref UInt32 Data);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatSdoRead(byte Slave, UInt16 Index, byte SubIndex, UInt16 size, int Timeout_ns, System.Text.StringBuilder str);


        #endregion
        #region EEPROM
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatEepromReq(UInt16 OP, UInt16 slave, UInt16 eeproma, UInt16 data, UInt32 timeout);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatEepromGet(ref UInt64 pu64Data);
        #endregion
        #region GPIO
        /******************************************************************************
         * GPIO_MODE Definitions
         ******************************************************************************/
        public const byte ECM_GPIO_MODE_INPUT = 0x00;/* Input Mode */
        public const byte ECM_GPIO_MODE_OUTPUT = 0x01;/* Output Mode */
        public const byte ECM_GPIO_MODE_OPEN_DRAIN = 0x02;/* Open-Drain Mode */
        public const byte ECM_GPIO_MODE_BIDIR = 0x03;/* Quasi-bidirectional Mode */

        /******************************************************************************
         * GPIO Interrupt Type Definitions
         ******************************************************************************/
        public const byte ECM_GPIO_INT_DIABLE = 0x00;/* Disable Interrupt */
        public const byte ECM_GPIO_INT_BOTH_EDGE = 0x03;/* Interrupt enable by both Rising Edge and Falling Edge */
        public const byte ECM_GPIO_INT_FALLING = 0x04;/* Interrupt enable by Input Falling Edge */
        public const byte ECM_GPIO_INT_RISING = 0x05;/* Interrupt enable by Input Rising Edge */

        /******************************************************************************
         * GPIO Pull Select Definitions
         ******************************************************************************/
        public const byte ECM_GPIO_PUSEL_DISABLE = 0x00;/* Pull Select Disable Mode */
        public const byte ECM_GPIO_PUSEL_PULL_UP = 0x01;/* Pull-up Mode */
        public const byte ECM_GPIO_PUSEL_PULL_DOWN = 0x02;/* Pull-down Mode */

        /// <summary>
        /// Set GPIO mode
        /// </summary>
        /// <param name="ch">GPIO channel. The valid range is from 0 to 15.</param>
        /// <param name="direction">IO direction. See ECM_GPIO_MODE_## definitions.</param>
        /// <param name="pusel">Pull select. See ECM_GPIO_PUSEL_## definitions.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GpioSetMode(byte ch, byte direction, byte pusel);

        /// <summary>
        /// Enable/Disable Debounce mode
        /// </summary>
        /// <param name="ch">GPIO channel. The valid range is from 0 to 15.</param>
        /// <param name="enable">1 for enable debounce; 0 for disable debounce.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GpioEnableDebounce(byte ch, byte enable);

        /// <summary>
        /// Set Debounce Clock
        /// </summary>
        /// <param name="source"> 1 for 10K clock source; 0 for 192M clock source</param>
        /// <param name="clock">Debounce cycle. The sampling cycle is 2^(@clock)*(clock source). The valid range is from 0 to 15.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GpioSetDebounceClock(byte source, byte clock);

        /// <summary>
        /// Set GPIO value.
        /// </summary>
        /// <param name="val">GPIO output value. The n-th bit of @val is mapping to the n-th GPIO channel.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GpioSetValue(UInt16 val);

        /// <summary>
        /// Get GPIO value.
        /// </summary>
        /// <param name="val">GPIO input value pointer. The n-th bit of *(@val) is mapping to the n-th GPIO channel.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GpioGetValue(ref UInt16 val);

        /// <summary>
        /// Enable/Disable GPIO Interrupt.
        /// </summary>
        /// <param name="ch">GPIO channel. The valid range is from 0 to 15.</param>
        /// <param name="inttype">Interrupt type. See ECM_GPIO_INT_## definitions.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GpioIntEnable(byte ch, byte inttype);

        /// <summary>
        /// Clear GPIO Interrupt flag.
        /// </summary>
        /// <param name="ch">Clear the interrupt flag of the GPIO channel. The valid range is from 0 to 15.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GpioIntClear(byte ch);

        /// <summary>
        /// Get GPIO Interrupt flag.
        /// </summary>
        /// <param name="val">GPIO Interrupt flag pointer. The n-th bit of *(@val) is mapping to the interrupt flag of the n-th GPIO channel.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GpioGetIntFlag(ref UInt16 val);

        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_SetEcatIntEnable(UInt32 u32CompIntEnable, byte u8GpioIntEnable, byte u8PeripIntEnable, byte u8INTActiveHigh);

        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetEcatIntEnable(ref UInt32 pu32CompIntEnable, ref byte pu8GpioIntEnable, ref byte pu8PeripIntEnable, ref byte pu8INTActiveHigh);

        #endregion
        #region DAC ADC
        /******************************************************************************
         * ADC_MODE Definitions
         ******************************************************************************/
        public const UInt32 ECM_ADC_MODE_SINGLE_END = 0;   /* Single-end input mode  */
        public const UInt32 ECM_ADC_MODE_DIFFERENTIAL = 1;   /* Differential input mode */

        /******************************************************************************
         * ADC_TRIG Definitions
         ******************************************************************************/
        public const UInt32 ECM_ADC_TRIG_SOFTWARE = 0;      /* Software trigger. Use ECM_AdcStartConv() function to trigger ADC input */

        /******************************************************************************
         * DAC_MODE Definitions
         ******************************************************************************/
        public const UInt32 ECM_DAC_TRIG_MODE_WRTTE_DATA = 0x0000;        /* Write data trigger. Use ECM_DacSetData() function to trigger DAC output. */
        public const UInt32 ECM_DAC_TRIG_MODE_SOFTWARE = 0x0010;        /* software trigger. Use ECM_DacStartConv() function to trigger DAC output. */
        public const UInt32 ECM_DAC_TRIG_MODE_LOW_LEVEL = 0x0030;        /* Low-level trigger. DAC is triggered if DAC_ST pin is low. */
        public const UInt32 ECM_DAC_TRIG_MODE_HIGH_LEVEL = 0x1030;        /* High-level trigger. DAC is triggered if DAC_ST pin is high. */
        public const UInt32 ECM_DAC_TRIG_MODE_RISING_EDGE = 0x2030;        /* Rising-edge trigger. DAC is triggered if DAC_ST pin detect a rising edge. */
        public const UInt32 ECM_DAC_TRIG_MODE_FALLING_EDGE = 0x3030;        /* Falling-edge trigger. DAC is triggered if DAC_ST pin detect a falling edge. */


        /// <summary>
        /// Open ADC device and Set ADC input mode
        /// </summary>
        /// <param name="mode">ADC mode. See ECM_ADC_MODE_## definitions.</param>
        /// <returns> 1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_AdcOpen(UInt32 mode);

        /// <summary>
        /// Close ADC device
        /// </summary>
        /// <returns>1 for success; 0 for timeout.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_AdcClose();

        /// <summary>
        /// Select ADC trigger source
        /// </summary>
        /// <param name="source">ADC trigger source. See ECM_ADC_TRIG_## definitions.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_AdcConfigSampleModule(UInt32 source);

        /// <summary>
        /// Get ADC data valid flag
        /// </summary>
        /// <param name="flag">ADC data valid flag. The data is valid if the flag is non-zero.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_AdcGetDataValidFlag(ref UInt32 flag);

        /// <summary>
        /// Start ADC convert.
        /// </summary>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_AdcStartConv();

        /// <summary>
        /// Get ADC data value
        /// </summary>
        /// <param name="data">ADC data value.</param>
        /// <returns> 1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_AdcGetConvData(ref UInt32 data);

        /// <summary>
        /// Open DAC device and Set DAC trigger mode
        /// </summary>
        /// <param name="mode">DAC mode. See ECM_DAC_TRIG_MODE_## definitions.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_DacOpen(UInt32 mode);

        /// <summary>
        /// Close DAC device.
        /// </summary>
        /// <returns>1 for success; 0 for timeout.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_DacClose();

        /// <summary>
        /// Set delay time for DAC to become stable.
        /// </summary>
        /// <param name="delay">Delay time in micro second. The range is 0 to 1023/96.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters. </returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_DacSetDelayTime(UInt32 delay);

        /// <summary>
        /// Set DAC data value
        /// </summary>
        /// <param name="data">DAC data value.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_DacSetData(UInt32 data);

        /// <summary>
        /// Start DAC convert.
        /// </summary>
        /// <returns>1 for success; 0 for timeout.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_DacStartConv();
        #endregion
        #region QEI (ENC)
        /******************************************************************************
        * ENC_MODE Definitions
        ******************************************************************************/
        public const byte ECM_ENC_MODE_X4_FREE = 0x0; /* X4 free-counting Mode */
        public const byte ECM_ENC_MODE_X2_FREE = 0x1; /* X2 free-counting Mode */
        public const byte ECM_ENC_MODE_X4_COMP = 0x2; /* X4 compare-counting Mode */
        public const byte ECM_ENC_MODE_X2_COMP = 0x3; /* X2 compare-counting Mode */

        /// <summary>
        /// Open and Set Encoder mode
        /// </summary>
        /// <param name="mode">Encoder mode. See ECM_ENC_MODE_## definitions.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EncOpen(byte mode);

        /// <summary>
        /// Start Encoder counting
        /// </summary>
        /// <returns>1 for success; 0 for timeout.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EncStart();

        /// <summary>
        /// Stop Encoder counting
        /// </summary>
        /// <returns>1 for success; 0 for timeout.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EncStop();

        /// <summary>
        /// Get Encoder counter value
        /// </summary>
        /// <param name="count">Encoder counter value pointer. The encoder counter value is filled in *(@count) if success.</param>
        /// <returns>1 for success; 0 for timeout; -1 for invalid input parameters.</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EncGetCount(ref UInt32 count);
        #endregion
        #region ObjectInfo
        public const UInt16 OTYPE_VAR = 0x0007;
        public const UInt16 OTYPE_ARRAY = 0x0008;
        public const UInt16 OTYPE_RECORD = 0x0009;

        public const byte ATYPE_Rpre = 0x01;
        public const byte ATYPE_Rsafe = 0x02;
        public const byte ATYPE_Rop = 0x04;
        public const byte ATYPE_Wpre = 0x08;
        public const byte ATYPE_Wsafe = 0x10;
        public const byte ATYPE_Wop = 0x20;

        public enum ec_datatype : UInt16
        {
            ECT_BOOLEAN = 0x0001,
            ECT_INTEGER8 = 0x0002,
            ECT_INTEGER16 = 0x0003,
            ECT_INTEGER32 = 0x0004,
            ECT_UNSIGNED8 = 0x0005,
            ECT_UNSIGNED16 = 0x0006,
            ECT_UNSIGNED32 = 0x0007,
            ECT_REAL32 = 0x0008,
            ECT_VISIBLE_STRING = 0x0009,
            ECT_OCTET_STRING = 0x000A,
            ECT_UNICODE_STRING = 0x000B,
            ECT_TIME_OF_DAY = 0x000C,
            ECT_TIME_DIFFERENCE = 0x000D,
            ECT_DOMAIN = 0x000F,
            ECT_INTEGER24 = 0x0010,
            ECT_REAL64 = 0x0011,
            ECT_INTEGER64 = 0x0015,
            ECT_UNSIGNED24 = 0x0016,
            ECT_UNSIGNED64 = 0x001B,
            ECT_BIT1 = 0x0030,
            ECT_BIT2 = 0x0031,
            ECT_BIT3 = 0x0032,
            ECT_BIT4 = 0x0033,
            ECT_BIT5 = 0x0034,
            ECT_BIT6 = 0x0035,
            ECT_BIT7 = 0x0036,
            ECT_BIT8 = 0x0037
        };
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_AllocODOEList();
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_ReadODList(byte slave);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_ReadOE(UInt16 item, byte subindex);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_ReadODDesc(UInt16 item);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetODEntries(ref UInt16 pu16);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetODIndex(ref UInt16 pu16, UInt16 entry);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetODDatatype(ref UInt16 pu16);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetODObjcode(ref byte pu8);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetODMaxSub(ref byte pu8);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetODName(System.Text.StringBuilder str);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetOEValueInfo(ref byte pu8);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetOEDataType(ref UInt16 pu16);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetOEBitLength(ref UInt16 pu16);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetOEObjAccess(ref UInt16 pu16);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_GetOEName(System.Text.StringBuilder str);  
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_FreeODOEList();
        #endregion
        #region FoE
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatFoEDownload(byte slave, string filename, UInt32 password, UInt32 filesize, UInt32 startaddr, UInt32 timeout);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatFoEUpload(byte slave, string filename, UInt32 password, ref UInt32 pFilesize, UInt32 startaddr, UInt32 timeout);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatFoEDataWrite(UInt32 addr, UInt32 datasize, byte[] pBuf);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatFoEDataRead(UInt32 addr, ref UInt32 pDatasize, byte[] pBuf);
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_EcatFoEDataGet(IntPtr pBuf);
        #endregion
        #region MDIO
        /// <summary>
        /// ECM_MDIOWrite (For version 0x1F or later)
        /// </summary>
        /// <param name="u8Reg">PHY register</param>
        /// <param name="u8Addr">PHY address</param>
        /// <param name="u16Data">write data</param>
        /// <returns>greater than zero for success</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_MDIOWrite(byte u8Reg, byte u8Addr, UInt16 u16Data);
        /// <summary>
        ///  ECM_MDIORead (For version 0x1F or later)
        /// </summary>
        /// <param name="u8Reg">PHY register</param>
        /// <param name="u8Addr">PHY address</param>
        /// <param name="pu16Data">read data pointer</param>
        /// <returns>greater than zero for success</returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_MDIORead(byte u8Reg, byte u8Addr, ref UInt16 pu16Data);
        #endregion

        #region ECM_DirectAssign
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern void ECM_SetEcatInitMode(byte u8mode);
        /// <summary>
        /// ECM_DirectAssign (For version 0x1F or later)
        /// </summary>
        /// <param name="u8Slv">slave ID</param>
        /// <param name="u8Mode">0 for disable direct assignment (default); 1 for enable direct assignment</param>
        /// <param name="u16Obits">output process data size</param>
        /// <param name="u16Ibits">input process data size</param>
        /// <returns></returns>
        [DllImport(DrvDllFilePath, CallingConvention = CallingConvention.Cdecl)]
        public static extern int ECM_DirectAssign(byte u8Slv, byte u8Mode, UInt16 u16Obits, UInt16 u16Ibits);        
        #endregion
        #endregion
        }
}
