﻿using System;
using static XFUDriver.XFUDriver;

namespace IO_CSharp
{
    partial class Program
    {
        static byte[] g_u8RxPDO = new byte[1400];
        static byte[] g_u8TxPDO = new byte[1400];
        static void Main(string[] args)
        {
            #region Init
            int nRet = 0;
            byte u8Version = 0, u8TmpEcState = 0, u8CrcErrCnt = 0, u8FifoCnt = 0, u8WkcErrCnt = 0, u8IsSlvAlive = 0;
            byte u8TxFIFOCnt = 64, u8RxFIFOCnt = 64;
            ushort u16RxPDOSize, u16TxPDOSize;
            uint u32InfoBuf = 0;
            sbyte nSlvCntOnline;
            int bLinkPin = -1;
            ushort u16SpiSize = 992;

            XF_SetDevType(1);   // 第一個參數為 0 for USB, 1 for PCIe

            nRet = UserSys_Init(24000000);
            if (nRet < 0)
            {
                Console.WriteLine("UserSys_Init return {0} Any key to exit.", nRet);
                Console.ReadKey();
                return;
            }
            u8Version = ECM_InitLibrary(ref u16SpiSize, ECM_CRC_TYPE.NONE);
            Console.WriteLine("u8Version 0x{0:X}", u8Version); //IC內部韌體版本
            if (u8Version == 0)
                return;
            nRet = ECM_GetNetLinkPin(ref bLinkPin);
            Console.WriteLine("ECM_GetNetLinkPin linkpin = {0}, ret={1}", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

            nRet = ECM_EcatInit(0x0, 1000000); // 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
            Console.WriteLine("ECM_EcatInit return {0}", nRet); //return 1 代表 Init成功
            #endregion

            #region get slave info and set fifo
            nSlvCntOnline = ECM_EcatSlvCntGet();
            Console.WriteLine("Found {0} slave(s)", nSlvCntOnline);
            for (byte i = 0; i < nSlvCntOnline; i++)
            {
                nRet = ECM_SlaveInfoGet(i, ECM_SLV_INFO.ALstatuscode, ref u32InfoBuf);
                if (nRet > 0) //nRet: Slave Information Size
                {
                    if (u32InfoBuf != 0)
                    {
                        Console.WriteLine("Slave:{0} AL StatusCode:{1}...", i, u32InfoBuf); //AL StatusCode 非0 代表該Slave存在錯誤
                    }
                }
                else
                {
                    Console.WriteLine("Slave {0} Get AL StatusCode failed...", i);
                }
            }

            ECM_SetTxFIFOCnt(u8TxFIFOCnt); //Slave -> Master
            ECM_SetRxFIFOCnt(u8RxFIFOCnt); //Master -> Slave
            ECM_InitFIFO();
            // RxPDOSize MUST be the same as RxPDO struct size
            u16RxPDOSize = ECM_FifoRxPdoSizeGet();
            Console.Write("RxPDOSize {0} Byte(s), RxPDO struct size {1}\n", u16RxPDOSize, System.Runtime.InteropServices.Marshal.SizeOf(cmdData));
            // TxPDOSize MUST be the same as TxPDO struct size
            u16TxPDOSize = ECM_FifoTxPdoSizeGet();
            Console.Write("TxPDOSize {0} Byte(s), TxPDO struct size {1}\n", u16TxPDOSize, System.Runtime.InteropServices.Marshal.SizeOf(respData));

            nRet = ECM_EcatStateUpdate();
            if (nRet <= 0)
            {
                Console.Write("ECM_EcatStateUpdate fail({0})\n", nRet);
            }
            else
            {
                Console.Write("ECM_EcatStateUpdate done\n");
            }
            for (byte i = 0; i < nSlvCntOnline; i++)
            {
                nRet = ECM_EcatStateGet(i, ref u8TmpEcState);
                if (nRet > 0)
                {
                    Console.Write("slave {0} EcState:0x{1:X}\n", i, u8TmpEcState); // u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
                }
            }

            Console.Write("Any key to continue\n");
            Console.ReadKey();
            #endregion
            #region Get Slave Reg Information
            UInt16 u16d = 0x1;
            UInt32 configadr = 0;
            ECM_SlaveInfoGet(0, ECM_SLV_INFO.configadr, ref configadr);

            Console.WriteLine("Configured Address Physical Read AL status");
            u16d = 0;
            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x300, 0, 2, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 0 RX Error Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x302, 0, 2, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 1 RX Error Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x304, 0, 2, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 2 RX Error Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x306, 0, 2, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 3 RX Error Cnt = 0x{0,02:x}", u16d);

            u16d = 0;
            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x308, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 0 Forwarded RX Error Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x309, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 1 Forwarded RX Error Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x30A, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 2 Forwarded RX Error Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x30B, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 3 Forwarded RX Error Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x30C, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("ECAT Processing Unit Error Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x310, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 0 Lost Link Cnt Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x311, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 1 Lost Link Cnt Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x312, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 2 Lost Link Cnt Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x313, 0, 1, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("Port 3 Lost Link Cnt Cnt = 0x{0,02:x}", u16d);

            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x110, 0, 2, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("DL Status = 0x{0,02:x}", u16d);

            Console.WriteLine("Any key to Broadcast Write AL request to Init.");
            Console.ReadKey();
            u16d = 1;
            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_BWR, 0, 0x120, 0, 2, 1000000, ref u16d);

            Console.WriteLine("Any key to Configured Address Physical Write AL request to PerOP");
            Console.ReadKey();
            u16d = 2;
            ECM_SlaveInfoGet(0, ECM_SLV_INFO.configadr, ref configadr);
            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPWR, (UInt16)configadr, 0x120, 0, 2, 1000000, ref u16d);

            Console.WriteLine("Any key to Configured Address Physical Read AL status");
            Console.ReadKey();
            u16d = 0;
            ECM_EcatDatagramReq(ecm_datagram_commad.ECM_RAW_API_FPRD, (UInt16)configadr, 0x130, 0, 2, 1000000, ref u16d);
            ECM_EcatDatagramGet(ref u16d);
            Console.WriteLine("AL status = 0x{0,02:x}", u16d);
            Console.WriteLine("============================\r\n Any key to finish.");
            Console.ReadKey();
            #endregion

            #region control state to Init and clsoe the system
            ECM_EcatStateSet(0xFF, EC_STATE_INIT);
            System.Threading.Thread.Sleep(100);
            ECM_CloseLibrary();
            UserSys_Close();
            #endregion
        }
        private static int SetPDOInfo(PDO_CONFIG_HEAD pdoInfo)
        {
            int nret;
            UInt32 u32Tmp;
            if (pdoInfo.SmaIdx != 0x1C12 && pdoInfo.SmaIdx != 0x1C13)
                return -1;
            // clear pdo count
            byte zero = 0;
            nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, 0, sizeof(byte), 10000000, ref zero);
            if (nret <= 0)
            {
                return -2;
            }
            for (byte nPdoIdx = 0; nPdoIdx < pdoInfo.PDOCnt; nPdoIdx++)
            {
                if (pdoInfo.ObjsCnt[nPdoIdx] > 0)
                {
                    nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], 0, sizeof(byte), 10000000, ref zero);
                    for (byte nObjIdx = 0; nObjIdx < pdoInfo.ObjsCnt[nPdoIdx]; nObjIdx++)
                    {
                        u32Tmp = (UInt32)((pdoInfo.Table[nPdoIdx, nObjIdx].u16Idx * 0x10000) + (pdoInfo.Table[nPdoIdx, nObjIdx].u8SubIdx * 0x100) + (pdoInfo.Table[nPdoIdx, nObjIdx].u8BitSize));
                        nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], (byte)(1 + nObjIdx), sizeof(UInt32), 10000000, ref u32Tmp);
                        if (nret <= 0)
                        {
                            return -3;
                        }
                    }
                    nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.MapIdx[nPdoIdx], 0, sizeof(byte), 10000000, ref pdoInfo.ObjsCnt[nPdoIdx]);
                    if (nret <= 0)
                    {
                        return -4;
                    }
                }
                nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, (byte)(1 + nPdoIdx), sizeof(UInt16), 10000000, ref pdoInfo.MapIdx[nPdoIdx]);
                if (nret <= 0)
                {
                    return -5;
                }

            }
            nret = ECM_EcatSdoWrite(pdoInfo.slave, pdoInfo.SmaIdx, 0, sizeof(byte), 10000000, ref pdoInfo.PDOCnt);
            if (nret <= 0)
            {
                return -6;
            }
            return 0;
        }
    }
}
