﻿/******************************************************************************
 *	File	:	DA_AD_QEI_LIO.cpp
 *	Version :	1.7
 *	Date	:	2022/07/27
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <windows.h>
#include <iostream>
#include <mutex>
#include <platform.h>
#include <EcmUsrDriver.h>
#include <EcmAdcDriver.h>
#include <EcmDacDriver.h>
#include <EcmEncDriver.h>
#include <EcmGpioDriver.h>
uint8_t g_u8RxPDO[1400];
uint8_t g_u8TxPDO[1400];

int main()
{
#pragma region Init
	std::mutex _lock;
	unsigned short u16SpiSize = 992;
	uint8_t u8Version = 0;
	int i = 0, nRet = 0;
	memset(g_u8RxPDO, 0, sizeof(g_u8RxPDO));
	memset(g_u8TxPDO, 0, sizeof(g_u8TxPDO));
	// 第一個參數為 0 for USB
	// 第一個參數為 1 for PCIe
	XF_SetDevType(0);
	// 初始化PCIe晶片
	//
	nRet = UserSys_Init(24000000);
	if (nRet < 0) {
		printf("UserSys_Init return %d\r\n", nRet);
		return -1;
	}
	u8Version = ECM_InitLibrary(&u16SpiSize, ECM_CRC_TYPE_NONE);
	printf("u8Version 0x%X\r\n", u8Version);
	if (u8Version == 0)
		return 0;
#pragma endregion
#pragma region LIO - Output
	printf("Set all GPIO pins on the chip as the output pin. Any key to set all outputs to HIGH.\r\n");
	getchar();
	for (byte i = 0; i < 16; i++)
	{
		nRet = ECM_GpioSetMode(i, ECM_GPIO_MODE_OUTPUT, ECM_GPIO_PUSEL_DISABLE);
		if (nRet < 0)
		{
			printf("GpioSetMode %d Failed\r\n", i);
			getchar();
		}
	}
	nRet = ECM_GpioSetValue(0xffff);//GPIO high
	if (nRet < 0)
	{
		printf("GpioSetValue Failed\r\n");
		getchar();
	}
	printf("Any key to set all outputs to LOW.\r\n");
	getchar();
	nRet = ECM_GpioSetValue(0x0);//GPIO low
	if (nRet < 0)
	{
		printf("GpioSetValue Failed\r\n");
		getchar();
	}
	printf("The output test finish. Any key to continue the input test.\r\n");
	getchar();
#pragma endregion
#pragma region LIO - Input / interrupt
	uint32_t u32CompInt = 0;
	uint8_t u8GPIOInt = 0, u8PeripInt = 0, u8ActiveHigh = 0;
	uint16_t inputValue = 0, intFlag = 0;
	printf("Set all GPIO pins on the chip as the input pin. \r\n");
	for (byte i = 0; i < 16; i++)
	{
		nRet = ECM_GpioSetMode(i, ECM_GPIO_MODE_INPUT, ECM_GPIO_PUSEL_PULL_DOWN);
		if (nRet < 0)
		{
			printf("GpioSetMode %d Failed.\r\n", i);
			getchar();
		}
		nRet = ECM_GpioIntEnable(i, ECM_GPIO_INT_FALLING);
		if (nRet < 0)
		{
			printf("GpioIntEnable %d Failed.\r\n", i);
			getchar();
		}
	}
	nRet = ECM_SetEcatIntEnable(0, 0xFF, 0, 0); //set GPIO0~7 as interrupt source
	if (nRet <= 0)
	{
		printf("ECM_SetEcatIntEnable Failed.\r\n");
		getchar();
	}
	nRet = ECM_GetEcatIntEnable(&u32CompInt, &u8GPIOInt, &u8PeripInt, &u8ActiveHigh);
	if (nRet < 0)
	{
		printf("ECM_GetEcatIntEnable failed.\r\n");
		getchar();
	}
	printf("Int1 u32CompInt = 0x%X, u8GPIOInt = 0x%X, u8PeripInt = 0x%X, u8ActiveHigh=0x%X\r\n", u32CompInt, u8GPIOInt, u8PeripInt, u8ActiveHigh);
	for (int countDown = 50000; countDown >= 0; countDown--)
	{
		nRet = ECM_GpioGetValue(&inputValue);
		if (nRet < 0)
		{
			printf("ECM_GpioGetValue failed\r\n");
			getchar();
		}
		nRet = ECM_GpioGetIntFlag(&intFlag);
		printf("Input value is 0x%X, intFlag = 0x%X    After %d counts finish.           \r", inputValue, intFlag, countDown);
		if (nRet < 0)
		{
			printf("GpioGetIntFlag failed.\r\n");
			getchar();
		}
		if ((countDown % 10000 == 0) & (intFlag != 0))
		{
			printf("\nClear ch");
			for (byte i = 0; i < 16; i++)
			{
				if ((intFlag & (1<<i)) != 0)
				{
					nRet = ECM_GpioIntClear(i);
					printf("%d, ", i);
				}
				if (nRet < 0)
				{
					printf("ECM_GpioIntClear failed\r\n");
					getchar();
				}
			}
			printf("\r\n");
		}
	}
#pragma endregion
#pragma region DAC / ADC
	//chip DAC & ADC test
	printf("Connect DAC pin to ADC pin. Any key to start ADC / DAC test.\r\n");
	getchar();
	nRet = ECM_DacOpen(ECM_DAC_TRIG_MODE_SOFTWARE);
	if (nRet < 0)
	{
		printf("DacOpen failed.\r\n");
		getchar();
	}
	nRet = ECM_AdcOpen(ECM_ADC_MODE_SINGLE_END);
	if (nRet < 0)
	{
		printf("AdcOpen failed.\r\n");
		getchar();
	}

	for (uint32_t i = 1; i < 257; i += 5)
	{
		uint32_t voltageScale = 0;
		uint32_t data = 0, valid = 0;

		voltageScale = 16 * i - 1;

		nRet = ECM_DacSetData(voltageScale);
		if (nRet < 0)
		{
			printf("DacSetData failed.\r\n");
			getchar();
		}
		nRet = ECM_DacSetDelayTime(1023);
		if (nRet < 0)
		{
			printf("DacSetDelayTime failed.\r\n");
			getchar();
		}
		nRet = ECM_DacStartConv();
		if (nRet < 0)
		{
			printf("DacStartConv failed.\r\n");
			getchar();
		}
		nRet = ECM_AdcConfigSampleModule(ECM_ADC_TRIG_SOFTWARE);
		if (nRet < 0)
		{
			printf("AdcConfigSampleModule failed.\r\n");
			getchar();
		}
		nRet = ECM_AdcStartConv();
		if (nRet < 0)
		{
			printf("AdcStartConv failed.\r\n");
			getchar();
		}

		do
		{
			nRet = ECM_AdcGetDataValidFlag(&valid);
			if (nRet < 0)
			{
				printf("AdcGatDataValidFlag failed.\r\n");
				getchar();
			}
		} while (valid == 0);

		nRet = ECM_AdcGetConvData(&data);
		if (nRet < 0)
		{
			printf("AdcGetConData failed.\r\n");
			getchar();
		}
		printf("DAC = %d, ADC = %d, reflash every 0.5 s    \r", voltageScale, data);//the value should be around the value is set as DacSetData
		Sleep(500);
	}
#pragma endregion
#pragma region QEI (ENC)
	//chip QEI (ENC) test
	printf("\nAny key to start QEI test.\r\n");
	getchar();
	nRet = ECM_EncOpen(ECM_ENC_MODE_X4_FREE);
	if (nRet < 0)
	{
		printf("EncOpen failed.\r\n");
		getchar();
	}
	else
	{
		printf("EncOpen success.\r\n");
	}
	nRet = ECM_EncStart();
	if (nRet < 0)
	{
		printf("EncStart failed.\r\n");
		getchar();
	}
	else
	{
		printf("EncStart success.\r\n");
	}
	uint32_t count = 0;
	for (int countDown = 120; countDown >= 0; countDown--)
	{

		nRet = ECM_EncGetCount(&count);
		if (nRet < 0)
		{
			printf("EncGetCount failed.\r\n");
			getchar();
		}
		else
		{
			printf("Count value is %d, reflash every 0.5 s. After %d counts finish.          \r", count, countDown);
			Sleep(500);
		}
	}
#pragma endregion
#pragma region Clsoe the system
	printf("\r\nAny key to close.\r\n");
	getchar();
	UserSys_Close();
#pragma endregion
}

// 執行程式: Ctrl + F5 或 [偵錯] > [啟動但不偵錯] 功能表
// 偵錯程式: F5 或 [偵錯] > [啟動偵錯] 功能表

// 開始使用的提示: 
//   1. 使用 [方案總管] 視窗，新增/管理檔案
//   2. 使用 [Team Explorer] 視窗，連線到原始檔控制
//   3. 使用 [輸出] 視窗，參閱組建輸出與其他訊息
//   4. 使用 [錯誤清單] 視窗，檢視錯誤
//   5. 前往 [專案] > [新增項目]，建立新的程式碼檔案，或是前往 [專案] > [新增現有項目]，將現有程式碼檔案新增至專案
//   6. 之後要再次開啟此專案時，請前往 [檔案] > [開啟] > [專案]，然後選取 .sln 檔案
