﻿/******************************************************************************
 *	File	:	EEPROM.cpp
 *	Version :	1.7
 *	Date	:	2022/07/27
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <windows.h>
#include <iostream>
#include <mutex>
#include <platform.h>
#include <EcmUsrDriver.h>

uint8_t g_u8RxPDO[1400];
uint8_t g_u8TxPDO[1400];

int main()
{
#pragma region Init
	std::mutex _lock;
	int bLinkPin;
	unsigned short u16SpiSize = 992;
	uint8_t u8Version = 0;
	int i = 0, nRet = 0;
	memset(g_u8RxPDO, 0, sizeof(g_u8RxPDO));
	memset(g_u8TxPDO, 0, sizeof(g_u8TxPDO));
	// 第一個參數為 0 for USB
	// 第一個參數為 1 for PCIe
	XF_SetDevType(0);
	// 初始化PCIe晶片
	//
	nRet = UserSys_Init(24000000);
	if (nRet < 0) {
		printf("UserSys_Init return %d\r\n", nRet);
		return -1;
	}
	u8Version = ECM_InitLibrary(&u16SpiSize, ECM_CRC_TYPE_NONE);
	printf("u8Version 0x%X\r\n", u8Version);
	if (u8Version == 0)
		return 0;
	nRet = ECM_GetNetLinkPin(&bLinkPin);
	printf("ECM_GetNetLinkPin linkpin(%d) ret(%d)\n", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

	nRet = ECM_EcatInit(0x0, 1000000);// 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
	printf("ECM_EcatInit %d\r\n", nRet); //return 1 代表 Init成功
#pragma endregion
#pragma region Read EEPROM
	const int EEP_SIZE = 512;
	uint8_t eeprom[EEP_SIZE];
	uint32_t *pu32EEP = (uint32_t *)eeprom;
	printf("uint8_t sii_data[] = \r\n{");

	for (i = 0; i < EEP_SIZE / 4; i++) {
		/*
		 * Two step to get SII data.
		 *      ECM_EcatEepromReq(ECM_EEPROM_OP_RD, slave, address, 0, pu8EcmStatus);
		 *      ECM_EcatEepromGet( pu64Data, pu8EcmStatus);
		 * Note that the address must be 32-bit aligned.
		 * This function get 32-bit data per request.
		 */
		uint32_t u32Data[2];
		ECM_EcatEepromReq(ECM_EEPROM_OP_RD, \
			0,/* slave */ \
			2 * i,/* eeprom addr */ \
			0,/* don't care if read op */ \
			700000000/*ns*/ \
		);
		nRet = ECM_EcatEepromGet((uint64_t *)u32Data);
		if (nRet <= 0)
			return -1;
		if ((i & 3) == 0)
			printf("\r\n");//PRINTF("\r\nADDR %04x:", 2*i);
		printf(" 0x%02x, 0x%02x, 0x%02x, 0x%02x,", ((uint8_t *)u32Data)[0], ((uint8_t *)u32Data)[1], ((uint8_t *)u32Data)[2], ((uint8_t *)u32Data)[3]);
		pu32EEP[i] = u32Data[0];
	}
	printf("\r\n};\r\n");
#pragma endregion
#pragma region Write EEPROM
	printf("Enter Y to write the EEPROM on slave. NOTE:The data will NOT be recovered!");
	if (89 == getchar())
	{
		uint8_t sii_data[] =
		{
		 0x05, 0x0e, 0x03, 0x44, 0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x52, 0x00,
		 0x10, 0x10, 0x10, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x10, 0x80, 0x00, 0x80, 0x10, 0x80, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x01, 0x00,
		 0x0a, 0x00, 0x49, 0x00, 0x0c, 0x05, 0x4e, 0x45, 0x58, 0x54, 0x57, 0x0c, 0x4e, 0x45, 0x58, 0x54,
		 0x57, 0x20, 0x50, 0x44, 0x49, 0x20, 0x49, 0x4f, 0x08, 0x53, 0x79, 0x6e, 0x63, 0x68, 0x72, 0x6f,
		 0x6e, 0x02, 0x44, 0x43, 0x07, 0x54, 0x78, 0x50, 0x44, 0x4f, 0x20, 0x30, 0x13, 0x44, 0x69, 0x67,
		 0x69, 0x74, 0x61, 0x20, 0x49, 0x6e, 0x70, 0x75, 0x74, 0x20, 0x42, 0x79, 0x74, 0x65, 0x20, 0x30,
		 0x07, 0x54, 0x78, 0x50, 0x44, 0x4f, 0x20, 0x31, 0x13, 0x44, 0x69, 0x67, 0x69, 0x74, 0x61, 0x20,
		 0x49, 0x6e, 0x70, 0x75, 0x74, 0x20, 0x42, 0x79, 0x74, 0x65, 0x20, 0x31, 0x07, 0x52, 0x78, 0x50,
		 0x44, 0x4f, 0x20, 0x30, 0x14, 0x44, 0x69, 0x67, 0x69, 0x74, 0x61, 0x20, 0x4f, 0x75, 0x74, 0x70,
		 0x75, 0x74, 0x20, 0x42, 0x79, 0x74, 0x65, 0x20, 0x30, 0x07, 0x52, 0x78, 0x50, 0x44, 0x4f, 0x20,
		 0x31, 0x14, 0x44, 0x69, 0x67, 0x69, 0x74, 0x61, 0x20, 0x4f, 0x75, 0x74, 0x70, 0x75, 0x74, 0x20,
		 0x42, 0x79, 0x74, 0x65, 0x20, 0x31, 0x1e, 0x00, 0x10, 0x00, 0x01, 0x00, 0x01, 0x02, 0x01, 0x27,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x01, 0x00, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x28, 0x00, 0x02, 0x00, 0x01, 0x02,
		 0x03, 0xff, 0x29, 0x00, 0x10, 0x00, 0x00, 0x10, 0x80, 0x00, 0x26, 0x00, 0x01, 0x01, 0x80, 0x10,
		 0x80, 0x00, 0x22, 0x00, 0x01, 0x02, 0x00, 0x11, 0x00, 0x00, 0x64, 0x00, 0x01, 0x03, 0x00, 0x14,
		 0x00, 0x00, 0x20, 0x00, 0x01, 0x04, 0x32, 0x00, 0x10, 0x00, 0x00, 0x1a, 0x01, 0x03, 0x00, 0x05,
		 0x10, 0x00, 0x00, 0x60, 0x01, 0x06, 0x05, 0x08, 0x00, 0x00, 0x01, 0x1a, 0x01, 0x03, 0x00, 0x07,
		 0x10, 0x00, 0x00, 0x60, 0x01, 0x08, 0x05, 0x08, 0x00, 0x00, 0x33, 0x00, 0x10, 0x00, 0x00, 0x16,
		 0x01, 0x02, 0x00, 0x09, 0x10, 0x00, 0x00, 0x70, 0x01, 0x0a, 0x05, 0x08, 0x00, 0x00, 0x01, 0x16,
		 0x01, 0x02, 0x00, 0x0b, 0x10, 0x00, 0x00, 0x70, 0x02, 0x0c, 0x05, 0x08, 0x00, 0x00, 0x3c, 0x00,
		 0x18, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x01, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x01, 0x00, 0x04, 0x00, 0x00, 0x00,
		 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
		 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
		};
		for (i = 0; i < sizeof(sii_data) / 2; i++) 
		{
			uint16_t data = sii_data[2 * i] + (sii_data[2 * i + 1] << 8);
			ECM_EcatEepromReq(ECM_EEPROM_OP_WR, \
				0,/* slave */ \
				i,/* eeprom addr */ \
				data,/* don't care if read op */ \
				700000000/*ns*/ \
			);
		}
		printf("Write EEPROM finish.\r\n");
	}
#pragma endregion
#pragma region Clsoe the system
	printf("\r\nAny key to close.\r\n");
	getchar();
	UserSys_Close();
#pragma endregion
}

// 執行程式: Ctrl + F5 或 [偵錯] > [啟動但不偵錯] 功能表
// 偵錯程式: F5 或 [偵錯] > [啟動偵錯] 功能表

// 開始使用的提示: 
//   1. 使用 [方案總管] 視窗，新增/管理檔案
//   2. 使用 [Team Explorer] 視窗，連線到原始檔控制
//   3. 使用 [輸出] 視窗，參閱組建輸出與其他訊息
//   4. 使用 [錯誤清單] 視窗，檢視錯誤
//   5. 前往 [專案] > [新增項目]，建立新的程式碼檔案，或是前往 [專案] > [新增現有項目]，將現有程式碼檔案新增至專案
//   6. 之後要再次開啟此專案時，請前往 [檔案] > [開啟] > [專案]，然後選取 .sln 檔案
