﻿/******************************************************************************
 *	File	:	GetObjectInfo.cpp
 *	Version :	1.7
 *	Date	:	2023/06/06
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <windows.h>
#include <iostream>
#include <mutex>
#include <platform.h>
#include <EcmUsrDriver.h>
#include "PdoDefine.h"

#define OTYPE_VAR               0x0007
#define OTYPE_ARRAY             0x0008
#define OTYPE_RECORD            0x0009

#define ATYPE_Rpre              0x01
#define ATYPE_Rsafe             0x02
#define ATYPE_Rop               0x04
#define ATYPE_Wpre              0x08
#define ATYPE_Wsafe             0x10
#define ATYPE_Wop               0x20
typedef enum
{
	ECT_BOOLEAN = 0x0001,
	ECT_INTEGER8 = 0x0002,
	ECT_INTEGER16 = 0x0003,
	ECT_INTEGER32 = 0x0004,
	ECT_UNSIGNED8 = 0x0005,
	ECT_UNSIGNED16 = 0x0006,
	ECT_UNSIGNED32 = 0x0007,
	ECT_REAL32 = 0x0008,
	ECT_VISIBLE_STRING = 0x0009,
	ECT_OCTET_STRING = 0x000A,
	ECT_UNICODE_STRING = 0x000B,
	ECT_TIME_OF_DAY = 0x000C,
	ECT_TIME_DIFFERENCE = 0x000D,
	ECT_DOMAIN = 0x000F,
	ECT_INTEGER24 = 0x0010,
	ECT_REAL64 = 0x0011,
	ECT_INTEGER64 = 0x0015,
	ECT_UNSIGNED24 = 0x0016,
	ECT_UNSIGNED64 = 0x001B,
	ECT_BIT1 = 0x0030,
	ECT_BIT2 = 0x0031,
	ECT_BIT3 = 0x0032,
	ECT_BIT4 = 0x0033,
	ECT_BIT5 = 0x0034,
	ECT_BIT6 = 0x0035,
	ECT_BIT7 = 0x0036,
	ECT_BIT8 = 0x0037
} ec_datatype;

char* dtype2string(uint16_t dtype, uint16_t bitlen)
{
	static char str[32] = { 0 };

	switch (dtype)
	{
	case ECT_BOOLEAN:
		sprintf_s(str, 32, "BOOLEAN");
		break;
	case ECT_INTEGER8:
		sprintf_s(str, 32, "INTEGER8");
		break;
	case ECT_INTEGER16:
		sprintf_s(str, 32, "INTEGER16");
		break;
	case ECT_INTEGER32:
		sprintf_s(str, 32, "INTEGER32");
		break;
	case ECT_INTEGER24:
		sprintf_s(str, 32, "INTEGER24");
		break;
	case ECT_INTEGER64:
		sprintf_s(str, 32, "INTEGER64");
		break;
	case ECT_UNSIGNED8:
		sprintf_s(str, 32, "UNSIGNED8");
		break;
	case ECT_UNSIGNED16:
		sprintf_s(str, 32, "UNSIGNED16");
		break;
	case ECT_UNSIGNED32:
		sprintf_s(str, 32, "UNSIGNED32");
		break;
	case ECT_UNSIGNED24:
		sprintf_s(str, 32, "UNSIGNED24");
		break;
	case ECT_UNSIGNED64:
		sprintf_s(str, 32, "UNSIGNED64");
		break;
	case ECT_REAL32:
		sprintf_s(str, 32, "REAL32");
		break;
	case ECT_REAL64:
		sprintf_s(str, 32, "REAL64");
		break;
	case ECT_BIT1:
		sprintf_s(str, 32, "BIT1");
		break;
	case ECT_BIT2:
		sprintf_s(str, 32, "BIT2");
		break;
	case ECT_BIT3:
		sprintf_s(str, 32, "BIT3");
		break;
	case ECT_BIT4:
		sprintf_s(str, 32, "BIT4");
		break;
	case ECT_BIT5:
		sprintf_s(str, 32, "BIT5");
		break;
	case ECT_BIT6:
		sprintf_s(str, 32, "BIT6");
		break;
	case ECT_BIT7:
		sprintf_s(str, 32, "BIT7");
		break;
	case ECT_BIT8:
		sprintf_s(str, 32, "BIT8");
		break;
	case ECT_VISIBLE_STRING:
		sprintf_s(str, 32, "VISIBLE_STR(%d)", bitlen);
		break;
	case ECT_OCTET_STRING:
		sprintf_s(str, 32, "OCTET_STR(%d)", bitlen);
		break;
	default:
		sprintf_s(str, 32, "dt:0x%4.4X (%d)", dtype, bitlen);
	}
	return str;
}
char* otype2string(uint16_t otype)
{
	static char str[32] = { 0 };

	switch (otype)
	{
	case OTYPE_VAR:
		sprintf_s(str, 32, "VAR");
		break;
	case OTYPE_ARRAY:
		sprintf_s(str, 32, "ARRAY");
		break;
	case OTYPE_RECORD:
		sprintf_s(str, 32, "RECORD");
		break;
	default:
		sprintf_s(str, 32, "ot:0x%4.4X", otype);
	}
	return str;
}
char* access2string(uint16_t access)
{
	static char str[32] = { 0 };

	sprintf_s(str, 32, "%s%s%s%s%s%s",
		((access & ATYPE_Rpre) != 0 ? "R" : "_"),
		((access & ATYPE_Wpre) != 0 ? "W" : "_"),
		((access & ATYPE_Rsafe) != 0 ? "R" : "_"),
		((access & ATYPE_Wsafe) != 0 ? "W" : "_"),
		((access & ATYPE_Rop) != 0 ? "R" : "_"),
		((access & ATYPE_Wop) != 0 ? "W" : "_"));
	return str;
}
void getPdoInfo(int slv)
{
	int i, j;
	uint16_t entries;
	ECM_AllocODOEList();
	ECM_ReadODList(slv);
	ECM_GetODEntries(&entries);

	printf(" CoE Object Description found, %d entries.\n", entries);
	for (i = 0; i < entries; i++)
	{
		uint16_t index;
		uint8_t objcode;
		uint8_t max_sub;
		char odname[41];
		char name[128] = { 0 };

		ECM_ReadODDesc(i);
		ECM_GetODIndex(&index, i);
		ECM_GetODObjcode(&objcode);
		ECM_GetODMaxSub(&max_sub);
		ECM_GetODName(odname);
		snprintf(name, sizeof(name) - 1, "\"%s\"", odname);
		if (objcode == OTYPE_VAR)
		{
			printf("0x%04x      %-40s      [%s]\n", index, name,
				otype2string(objcode));
		}
		else
		{
			printf("0x%04x      %-40s      [%s  maxsub(0x%02x)]\n",
				index, name, otype2string(objcode),
				max_sub);
		}

		for (j = 0; j < max_sub + 1; j++)
		{
			uint16_t bitlength, objaccess, datatype;
			ECM_ReadOE(i, j);
			ECM_GetOEDataType(&datatype);
			ECM_GetOEBitLength(&bitlength);
			ECM_GetOEObjAccess(&objaccess);
			ECM_GetOEName(odname);
			if ((datatype > 0) && (bitlength > 0))
			{
				snprintf(name, sizeof(name) - 1, "\"%s\"", odname);
				printf("    0x%02x      %-40s      [%-16s %6s]\n", j, name,
					dtype2string(datatype, bitlength), access2string(objaccess));
			}
		}
	}
	ECM_FreeODOEList();
}

uint8_t g_u8RxPDO[1400];
uint8_t g_u8TxPDO[1400];
int main()
{
#pragma region Init
	std::mutex _lock;
	int bLinkPin;
	unsigned short u16SpiSize = 992;
	uint8_t u8State = 0, u8Version = 0, u8TmpEcState = 0, u8CrcErrCnt = 0, u8LastCrcErrCnt = 0, u8FifoCnt = 0;
	uint8_t u8TxFIFOCnt = 64, u8RxFIFOCnt = 64;
	uint16_t u16InfoBuf, u16RxPDOSize, u16TxPDOSize;
	TOTAL_RXPDO_ST_DEF_T	*pRxPDOData;
	TOTAL_TXPDO_ST_DEF_T	*pTxPDOData;
	int i = 0, nRet = 0;
	int nSlvCntOnline;
	memset(g_u8RxPDO, 0, sizeof(g_u8RxPDO));
	memset(g_u8TxPDO, 0, sizeof(g_u8TxPDO));
	pTxPDOData = (TOTAL_TXPDO_ST_DEF_T *)g_u8TxPDO;
	pRxPDOData = (TOTAL_RXPDO_ST_DEF_T *)g_u8RxPDO;
	// 第一個參數為 0 for USB
	// 第一個參數為 1 for PCIe
	XF_SetDevType(0);
	// 初始化PCIe晶片
	//
	nRet = UserSys_Init(24000000);
	if (nRet < 0) {
		printf("UserSys_Init return %d\r\n", nRet);
		return -1;
	}
	u8Version = ECM_InitLibrary(&u16SpiSize, ECM_CRC_TYPE_NONE);
	printf("u8Version 0x%X\r\n", u8Version);
	if (u8Version == 0)
		return 0;
	nRet = ECM_GetNetLinkPin(&bLinkPin);
	printf("ECM_GetNetLinkPin linkpin(%d) ret(%d)\r\n", bLinkPin, nRet); //linkpin=0 代表網路接口沒有連結

	nRet = ECM_EcatInit(0x0, 1000000); // 第一個參數0x300代表DC Sync, 0代表Free Run (依從站ESI說明設置), 第二個參數為週期時間(單位ns)
	printf("ECM_EcatInit %d\r\n", nRet); //return 1 代表 Init成功
#pragma endregion
#pragma region get slave info and set fifo
	nSlvCntOnline = ECM_EcatSlvCntGet();
	printf("Slave %d\n", nSlvCntOnline);
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_SlaveInfoGet(i, ECM_SLV_INFO_ALstatuscode, (uint8_t *)&u16InfoBuf);
		if (nRet > 0) {
			if (u16InfoBuf != 0) {
				printf("Slave %d AL StatusCode(%d)...\r\n", i, u16InfoBuf);
			}
		}
		else {
			printf("Slave %d Get AL StatusCode failed...\r\n", i);
		}
	}
	ECM_SetTxFIFOCnt(u8TxFIFOCnt); //Slave -> Master
	ECM_SetRxFIFOCnt(u8RxFIFOCnt); //Master -> Slave
	ECM_InitFIFO();

	// RxPDOSize MUST be the same as RxPDO struct size
	u16RxPDOSize = ECM_FifoRxPdoSizeGet();
	printf("RxPDOSize %d Byte(s), RxPDO struct size %zd Byte(s)\r\n", u16RxPDOSize, sizeof(Total_rxpdo_st_def_t));
	// TxPDOSize MUST be the same as TxPDO struct size
	u16TxPDOSize = ECM_FifoTxPdoSizeGet();
	printf("TxPDOSize %dByte(s), TxPDO struct size %zd Byte(s)\r\n", u16TxPDOSize, sizeof(Total_txpdo_st_def_t));

	nRet = ECM_EcatStateUpdate();
	if (nRet <= 0) {
		printf("ECM_EcatStateUpdate fail(%d)\r\n", nRet);
	}
	else {
		printf("ECM_EcatStateUpdate done.\r\n");
	}
	for (i = 0; i < nSlvCntOnline; i++) {
		nRet = ECM_EcatStateGet(i, &u8TmpEcState);
		if (nRet) {
			printf("(%d) EcState(0x%X)\r\n", i, u8TmpEcState);// u8TmpEcState: 1 Init, 2 preOP, 4 SafeOP, 8 OP
		}
	}
	printf("Any key to continue.");
	getchar();
#pragma endregion
#pragma region Get Slave PDO Information
	getPdoInfo(0); // get slave 0 PDO information
	printf("============================\r\n Any key to finish.");
	getchar();
#pragma endregion

#pragma region control state to Init and clsoe the system
	ECM_EcatStateSet(0xFF, EC_STATE_INIT);
	Sleep(100);
	ECM_CloseLibrary();
	UserSys_Close();
#pragma endregion
}

// 執行程式: Ctrl + F5 或 [偵錯] > [啟動但不偵錯] 功能表
// 偵錯程式: F5 或 [偵錯] > [啟動偵錯] 功能表

// 開始使用的提示: 
//   1. 使用 [方案總管] 視窗，新增/管理檔案
//   2. 使用 [Team Explorer] 視窗，連線到原始檔控制
//   3. 使用 [輸出] 視窗，參閱組建輸出與其他訊息
//   4. 使用 [錯誤清單] 視窗，檢視錯誤
//   5. 前往 [專案] > [新增項目]，建立新的程式碼檔案，或是前往 [專案] > [新增現有項目]，將現有程式碼檔案新增至專案
//   6. 之後要再次開啟此專案時，請前往 [檔案] > [開啟] > [專案]，然後選取 .sln 檔案
